#ifndef Fl_Device_H
#define Fl_Device_H
#include <FL/Enumerations.H>
#include <FL/Fl_Group.H>
#include <FL/Fl_Pixmap.H>
#include <FL/Fl_Bitmap.H>


extern FL_EXPORT Fl_Color fl_color_;
extern FL_EXPORT int fl_size_;
extern FL_EXPORT int fl_font_;

typedef void (*Fl_Draw_Image_Cb)(void* ,int,int,int,uchar*);


class FL_EXPORT Fl_Device{
public:
	virtual   void draw(Fl_Widget *w){w->redraw();};
	virtual   void color(Fl_Color color);
	virtual   void color(uchar r, uchar g, uchar b);
	virtual Fl_Color color(){return fl_color_;};
		 
	virtual   void push_clip(int x, int y, int w, int h);
protected:
//	virtual void clip(double x, double y, double w, double h){clip((int)(x+.5),(int)(y+.5),(int)(w+.5),(int)(h+.5));};
public:

	virtual   void push_no_clip();
	virtual   void pop_clip();
	 
	virtual   int not_clipped(int x, int y, int w, int h);

protected:
//	virtual int not_clipped(double x, double y, double w, double h){return not_clipped((int)(x+.5), (int)(y+.5), (int)(w+.5), (int)(h+.5));};

public:
	virtual   int clip_box(int _x, int _y, int _w, int _h, int& x, int& y, int& w, int& h);

protected:
//	virtual int clip_box(double _x, double _y, double _w, double _h, double& x, double& y, double& w, double& h){
//		int x_,y_,w_,h_,r;
//		r=clip_box((int)(_x+.5), (int)(_y+.5), (int)(_w+.5), (int)(_h+.5), x_, y_, w_, h_);
//		x=x_;y=y_;w=w_;h=h_;
//		return r;
//	};

public:
	virtual   void point(int x, int y);
	virtual   void line_style(int style, int width, char* dashes);

	virtual   void rect(int x, int y, int w, int h);
	virtual   void rectf(int x, int y, int w, int h);

	virtual   void line(int x1, int y1, int x2, int y2);
	virtual   void line(int x1, int y1, int x2, int y2, int x3, int y3);

	virtual   void loop(int x1, int y1, int x2 ,int y2, int x3, int y3);
	virtual   void loop(int x1, int y1, int x2 ,int y2, int x3, int y3, int x4, int y4);

	virtual   void polygon(int x1, int y1, int x2 ,int y2, int x3, int y3);
	virtual   void polygon(int x1 ,int y1, int x2, int y2, int x3, int y3, int x4, int y4);

	virtual   void xyline(int x, int y, int x1);
	virtual   void xyline(int x, int y, int x1, int y2);
	virtual   void xyline(int x, int y, int x1, int y2, int x3);

	virtual   void yxline(int x, int y, int y1);
	virtual   void yxline(int x, int y, int y1, int x2);
	virtual   void yxline(int x, int y, int y1, int x2, int y3);

	virtual   void arc(int x, int y, int w, int h, double a1, double a2);
	virtual   void pie(int x, int y, int w, int h, double a1, double a2);

	virtual   void begin_points();
	virtual   void begin_line();
	virtual   void begin_loop();
	virtual   void begin_polygon();
	virtual   void vertex(double x, double y);
	virtual   void curve(double x, double y, double x1, double y1, double x2, double y2, double x3, double y3);
	virtual   void arc(double x, double y, double r, double start, double a);
	virtual   void circle(double x, double y, double r);
	virtual   void end_points();
	virtual   void end_line();
	virtual   void end_loop();
	virtual   void end_polygon();
	virtual   void begin_complex_polygon();
	virtual   void gap();
	virtual   void end_complex_polygon();
	virtual   void transformed_vertex(double x, double y);

	virtual   void font(int face, int size);
	virtual int font(){return fl_font_;};
	virtual int size(){return fl_size_;};
	virtual   int height();
	virtual   double width(const char* s);
	virtual   double width(const char* s, int n);
	virtual   double width(uchar c);
	virtual   int descent();
	virtual   void draw(const char* s, int x, int y);
	virtual   void draw(const char* s, int n, int x, int y);
	virtual void transformed_draw(const char* s, int n, double x, double y){draw(s,n,(int)(x+.5),(int)(y+.5));};
	virtual void transformed_draw(const char* s, double x, double y){draw(s,(int)(x+.5),(int)(y+.5));};

	virtual   void draw_image(const uchar*, int,int,int,int, int delta=3, int ldelta=0);
	virtual   void draw_image_mono(const uchar*, int,int,int,int, int delta=1, int ld=0);
	virtual   void draw_image(Fl_Draw_Image_Cb, void*, int,int,int,int, int delta=3);
	virtual   void draw_image_mono(Fl_Draw_Image_Cb, void*, int,int,int,int, int delta=1);
	virtual   void rectf(int x, int y, int w, int h, uchar r, uchar g, uchar b);
	virtual   void draw_Pixmap(Fl_Pixmap * pxm,int XP, int YP, int WP, int HP, int cx, int cy);
	virtual   void draw_RGB(Fl_RGB_Image * rgb,int XP, int YP, int WP, int HP, int cx, int cy);
	virtual   void draw_Bitmap(Fl_Bitmap * bmp,int XP, int YP, int WP, int HP, int cx, int cy);
	virtual   ~Fl_Device(){};


};
extern FL_EXPORT Fl_Device fltk;
extern FL_EXPORT Fl_Device *fl;

#endif
