//
// "$Id: Fl_get_key.cxx,v 1.5.2.3.2.3 2002/01/01 15:11:31 easysw Exp $"
//
// Keyboard state routines for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2002 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "fltk-bugs@fltk.org".
//

#ifdef WIN32
#  include "Fl_get_key_win32.cxx"
#elif defined(__APPLE__)
#  include "Fl_get_key_mac.cxx"
#else

// Return the current state of a key.  This is the X version.  I identify
// keys (mostly) by the X keysym.  So this turns the keysym into a keycode
// and looks it up in the X key bit vector, which Fl_x.cxx keeps track of.

#  include <FL/Fl.H>
#  include <FL/x.H>

extern char fl_key_vector[32]; // in Fl_x.cxx

int Fl::event_key(int k) {
  if (k > FL_Button && k <= FL_Button+8)
    return Fl::event_state(8<<(k-FL_Button));
  int i;
#  ifdef __sgi
  // get some missing PC keyboard keys:
  if (k == FL_Meta_L) i = 147;
  else if (k == FL_Meta_R) i = 148;
  else if (k == FL_Menu) i = 149;
  else
#  endif
    i = XKeysymToKeycode(fl_display, k);
    i = fl_key_vector[i/8] & (1 << (i%8));
// COW - Added my own keystroke routine for the num keypad keys.
    i = CowEventKey(i);
    return(i);
}

int Fl::get_key(int k) {
  fl_open_display();
  XQueryKeymap(fl_display, fl_key_vector);
  return event_key(k);
}

int     CowEventKey(int n)
{
        if(Fl::event_state(FL_NUM_LOCK) != FL_NUM_LOCK)
        {
                if(n == (FL_KP + '1')) n = FL_End;
                if(n == (FL_KP + '2')) n = FL_Down;
                if(n == (FL_KP + '3')) n = FL_Page_Down;
                if(n == (FL_KP + '4')) n = FL_Left;
                if(n == (FL_KP + '6')) n = FL_Right;
                if(n == (FL_KP + '7')) n = FL_Home;
                if(n == (FL_KP + '8')) n = FL_Up;
                if(n == (FL_KP + '9')) n = FL_Page_Up;
                if(n == (FL_KP + '0')) n = FL_Insert;
                if(n == (FL_KP + '.')) n = FL_Delete;
        }
        else
        {
                if(n == (FL_KP + '1')) n = '1';
                if(n == (FL_KP + '2')) n = '2';
                if(n == (FL_KP + '3')) n = '3';
                if(n == (FL_KP + '4')) n = '4';
                if(n == (FL_KP + '5')) n = '5';
                if(n == (FL_KP + '6')) n = '6';
                if(n == (FL_KP + '7')) n = '7';
                if(n == (FL_KP + '8')) n = '8';
                if(n == (FL_KP + '9')) n = '9';
                if(n == (FL_KP + '0')) n = '0';
                if(n == (FL_KP + '.')) n = '.';
        }
        if(n == FL_KP_Enter) n = FL_Enter;
        return(n);
}


#endif

//
// End of "$Id: Fl_get_key.cxx,v 1.5.2.3.2.3 2002/01/01 15:11:31 easysw Exp $".
//
