//
// "$Id: fl_draw.cxx,v 1.6.2.4.2.12 2002/05/16 12:47:43 easysw Exp $"
//
// Label drawing code for the Fast Light Tool Kit (FLTK).
//
// Copyright 1998-2002 by Bill Spitzak and others.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "fltk-bugs@fltk.org".
//

// Implementation of fl_draw(const char*,int,int,int,int,Fl_Align)
// Used to draw all the labels and text, this routine:
// Word wraps the labels to fit into their bounding box.
// Breaks them into lines at the newlines.
// Expands all unprintable characters to ^X or \nnn notation
// Aligns them against the inside of the box.

// #define min(a,b) ((a)<(b)?(a):(b))
#include <FL/fl_draw.H>
#include <FL/Fl_Image.H>

#include "flstring.h"
#include <ctype.h>

#define MAXBUF 1024



#include <FL/Fl_Device.H>
Fl_Device fltk;
Fl_Device * fl=&fltk;

#include "fl_draw_generic.cxx"

#include "Fl_Printer.cxx"
#include "Fl_PostScript.cxx"


#ifdef WIN32
#include "Fl_Gdi.cxx"
#endif

void Fl_Device::draw_Bitmap(Fl_Bitmap * im, int X, int Y, int W, int H, int cx, int cy)
{
	im->draw_screen(X, Y, W, H, cx, cy);
}

void Fl_Device::draw_Pixmap(Fl_Pixmap * im, int X, int Y, int W, int H, int cx, int cy)
{
	im->draw_screen(X, Y, W, H, cx, cy);
}

void Fl_Device::draw_RGB(Fl_RGB_Image * im, int X, int Y, int W, int H, int cx, int cy)
{
	im->draw_screen(X, Y, W, H, cx, cy);
}

void Fl_Bitmap::draw(int X, int Y, int W, int H, int cx, int cy)
{
	fl->draw_Bitmap(this, X, Y, W, H, cx, cy);
}

void Fl_Pixmap::draw(int X, int Y, int W, int H, int cx, int cy)
{
	fl->draw_Pixmap(this, X, Y, W, H, cx, cy);
}

void Fl_RGB_Image::draw(int X, int Y, int W, int H, int cx, int cy)
{
	fl->draw_RGB(this, X, Y, W, H, cx, cy);
}
